import time
import json
from re import search
from requests import post
from os import environ as env
from . import metrics


def track(tracker_url=None):

    # Get version and repository URL from 'setup.py'
    version = None
    repo_url = None
    try:
        with open('setup.py') as setup:
            setup_file = setup.read()
            try:
                version_regex = r'version=[\'\"][\d.]+[\'\"]'
                version = search(version_regex, setup_file).group(0)[9:-1]
            except AttributeError:
                print ("No version available")
            try:
                url_regex = r'url=[\'\"]https?:\/\/(www\.)?' \
                            r'[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}' \
                            r'\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)[\'\"]'
                repo_url = search(url_regex, setup_file).group(0)[5:-1]
            except AttributeError:
                print ("No version available")
    except IOError as e:
        print ("No setup.py file exists for the current app")

    event = dict()
    event['date_sent'] = time.strftime("%Y-%m-%dT%H:%M:%S.000Z", time.gmtime())
    event['runtime'] = 'python'
    journey_metric = metrics.getJson()

    if env.get('VCAP_APPLICATION') is not None:
        vcap_app = json.loads(env['VCAP_APPLICATION'])
        #event = dict()
        #event['date_sent'] = time.strftime("%Y-%m-%dT%H:%M:%S.000Z", time.gmtime())
        if version is not None:
            event['code_version'] = version
        if repo_url is not None:
            event['repository_url'] = repo_url
        #event['runtime'] = 'python'
        event['application_name'] = str(vcap_app['name'])
        event['application_id'] = str(vcap_app['application_id'])
        event['instance_index'] = vcap_app['instance_index']
        event['space_id'] = str(vcap_app['space_id'])
        event['application_version'] = str(vcap_app['application_version'])
        event['application_uris'] = [str(uri) for uri in vcap_app['application_uris']]

        # Check for VCAP_SERVICES env var with at least one service
        # Refer to http://docs.cloudfoundry.org/devguide/deploy-apps/environment-variable.html#VCAP-SERVICES
        if env.get('VCAP_SERVICES') is not None and json.loads(env['VCAP_SERVICES']):
            vcap_services = json.loads(env['VCAP_SERVICES'])
            event['bound_vcap_services'] = dict()

            # For each bound service, count the number of instances and identify used plans
            for service in vcap_services:
                event['bound_vcap_services'][service] = {
                    'count': len(vcap_services[service]),
                    'plans': []
                }

                # Append plans for each instance
                for instance in vcap_services[service]:
                    if 'plan' in instance.keys():
                        event['bound_vcap_services'][service]['plans'].append(str(instance['plan']))

                if len(event['bound_vcap_services'][service]['plans']) == 0:
                    del event['bound_vcap_services'][service]['plans']

    if journey_metric is not None:
        event = metrics.massage(journey_metric, event)

    # Create and format request to Deployment Tracker
    url = 'https://metrics-tracker.mybluemix.net/api/v1/track' if tracker_url is None else tracker_url
    headers = {'content-type': "application/json"}
    try:
        response = post(url, data=json.dumps(event), headers=headers)
        print ('Uploaded stats: %s' % response.text)
    except Exception as e:
        print ('Deployment Tracker upload error: %s' % str(e))

def DSX(metric=None):
    # Skip if there is no metric
    if not metric:
        print ('No metrics data available')
    else:
        metrics.DSXtrack(metric)




"""
A Python client for CF app deployment tracking

See:
https://github.com/IBM-Bluemix/cf-deployment-tracker-service
https://github.com/IBM-Bluemix/cf-deployment-tracker-client-python
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='metrics-tracker-client',
    version='1.0.6',
    description='Python client for the deployment tracker',
    long_description=long_description,
    url='https://github.com/IBM/metrics-collector-client-python',
    author='Jake Peyser, Tommy Li',
    author_email='jepeyser@us.ibm.com, Tommy.chaoping.li@ibm.com',
    license='Apache-2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    keywords='demos samples metrics',
    packages=find_packages(exclude=['tests']),
    install_requires=['requests>=2','pyyaml'],
)



import json
import yaml
import time
import getpass
from requests import post, get

def getJson():
	try:
		f = open("repository.yaml", "r")
		response = f.read()
		original = yaml.load(response)
		data = json.loads(json.dumps(original))
		f.close()
		return data
	except:
		return None

def massage(journey_metric, event):
	try:
		event['config'] = {}
		event['config']['repository_id'] = journey_metric['id'] if journey_metric.get("id") else ''
		event['config']['target_runtimes'] = journey_metric['runtimes'] if journey_metric.get("runtimes") else ''
		event['config']['target_services'] = journey_metric['services'] if journey_metric.get("services") else ''
		event['config']['event_id'] = journey_metric['event_id'] if journey_metric.get("event_id") else ''
		event['config']['event_organizer'] = journey_metric['event_organizer'] if journey_metric.get("event_organizer") else ''
	except:
		pass
	return event

def DSXtrack(metric):
	event = dict()
	event['date_sent'] = time.strftime("%Y-%m-%dT%H:%M:%S.000Z", time.gmtime())
	event['runtime'] = 'python'
	try:
		event['space_id'] = getpass.getuser()
	except:
		pass
	try:
		link = ''.join(['https://raw.githubusercontent.com/', metric,'/master/repository.yaml'])
		response = get(link)
		original = yaml.load(response.text)
		data = json.loads(json.dumps(original))
		massage(data,event)
	except:
		print("metric not tracked")
		event['config'] = {}

	url = 'https://metrics-tracker.mybluemix.net/api/v1/track'
	headers = {'content-type': "application/json"}
	try:
		response = post(url, data=json.dumps(event), headers=headers)
	except Exception as e:
		print ('Deployment Tracker upload error: %s' % str(e))